import { Inject, Optional, Self, SkipSelf } from '@angular/core';
import { TokenNameProvider, moqInjectorProvidersFactory as moqInjectorProvidersFactory$1, Reflector } from 'ng-auto-moq';

class JestReflector {
    constructor(tokenNameProvider) {
        this.tokenNameProvider = tokenNameProvider;
        this.toParameter = ({ type, decorators }) => {
            let visibility = 0 /* None */;
            let token = type;
            let optional = false;
            for (const { type: decorator, args } of (decorators || [])) {
                if (decorator === Inject) {
                    token = args[0];
                }
                else if (decorator === Optional) {
                    optional = true;
                }
                else if (decorator === Self) {
                    visibility = 1 /* Self */;
                }
                else if (decorator === SkipSelf) {
                    visibility = 2 /* SkipSelf */;
                }
            }
            return {
                token: token,
                optional,
                visibility,
                displayName: this.tokenNameProvider.getName(token)
            };
        };
        return this.factory();
    }
    factory() {
        return (type) => {
            if ((type.ctorParameters instanceof Function) === false) {
                throw new Error(`Type should have a static 'ctorParameters' property containing downleveled decorator information.
https://github.com/thymikee/jest-preset-angular/blob/ac9b689d9ba18b5ae2e18e9ed083d2cbbf86b2fd/src/transformers/downlevel-ctor.ts`);
            }
            const params = type.ctorParameters();
            return params.map(this.toParameter);
        };
    }
}
JestReflector.ctorParameters = () => [
    { type: TokenNameProvider, decorators: [{ type: Inject, args: [TokenNameProvider,] }] }
];

/**
 * Provides the service-to-test and its (moq) dependencies
 */
const moqInjectorProvidersFactory = (options = {}) => moqInjectorProvidersFactory$1({
    providers: [
        { provide: Reflector, useClass: JestReflector, deps: [TokenNameProvider] },
        ...(options.providers || [])
    ]
});

/*
 * Public API Surface of ng-auto-moq/jest
 */

/**
 * Generated bundle index. Do not edit.
 */

export { JestReflector, moqInjectorProvidersFactory };
//# sourceMappingURL=ng-auto-moq-jest.js.map
